bl_info = {
    "name": "Visibility handler",
    "author": "Daniel Bystedt",
    "version": (1, 1),
    "blender": (4, 2, 0),
    "location": "View3D > View > Visibility handler ",
    "description": "Sets visibility of objects etc based on viewport or render",
    "warning": "",
    "doc_url": "",
    "category": "Interface",
}


import bpy
from bpy.props import BoolProperty, EnumProperty



class OBJECT_OT_visibility_handler(bpy.types.Operator):
    """Handle visiblity"""
    bl_idname = "object.visibility_handler"
    bl_label = "Visibility handler"
    bl_options = {'REGISTER', 'UNDO'}

    collections: BoolProperty(
        name = "Collections",
        description = "If collection visibility should be affected",
        default = True,
    )

    objects: BoolProperty(
        name = "Objects",
        description = "If objects visibility should be affected",
        default = True,
    )


    modifiers: BoolProperty(
        name = "Modifiers",
        description = "If object modifiers visibility should be affected",
        default = True,
    )


    modifier_properties: BoolProperty(
        name = "Modifier properties",
        description = "If modifier properties that affects viewport and/or rendering should be affected",
        default = False,
    )

    scope: EnumProperty(
        name="Scope",
        description="Choose which objects should be affected",
        items=(
            ('SELECTED_OBJECTS', "Selected objects", "Handle visibility on selected objects"),
            ('ACTIVE_VIEW_LAYER', "Active view layer", "Handle visibility on objects in the active view layer"),
            ('ACTIVE_SCENE', "Active scene", "Handle visibility on objects in the active scenes all view layers"),
            ('ALL_SCENES', "All scenes", "Handle visibility on all objects in all scenes"),
        ),
        default='ACTIVE_SCENE',
    )

    copy_settings: EnumProperty(
        name="Copy settings",
        description="Copy visibility settings from viewport to render or vice versa",
        items=(
            ('VIEWPORT_TO_RENDER', "Viewport to render", "Copy visibility from viewport to render settings"),
            ('RENDER_TO_VIEWPORT', "Render to viewport", "Copy visibility from render to visibility settings"),
        ),
        default='VIEWPORT_TO_RENDER',
    )


    def execute(self, context):

        print("\nEXECUTE ==================")
        if self.collections:
            handle_collections_visibility(context, self.copy_settings, self.scope)
        if self.objects:
            handle_object_visibility(context, self.copy_settings, self.scope)
        if self.modifiers:
            handle_modifier_visibility(context, self.copy_settings, self.scope)
        if self.modifier_properties:
            handle_modifier_properties(context, self.copy_settings, self.scope)
        return {'FINISHED'}

def get_view_layers(context, scope):

    '''
    Get view layers based on the scope
    '''

    view_layers = []

    if scope == 'ACTIVE_VIEW_LAYER' or scope == 'SELECTED_OBJECTS':
        return [context.view_layer]

    elif scope == 'ACTIVE_SCENE':
        for view_layer in context.scene.view_layers:
            view_layers.append(view_layer)
        return view_layers
    
    elif scope == 'ALL_SCENES':    
        for scene in bpy.data.scenes:
            for view_layer in scene.view_layers:
                view_layers.append(view_layer)
        return view_layers

def get_scenes(context, scope):
    '''
    Get scenes based on the scope
    '''    

    if scope == 'ALL_SCENES':    
        return bpy.data.scenes
    else:
        return [context.scene]

def get_objects(context, scope):
    '''
    Get objects within the defined scope
    '''
    if scope == 'SELECTED_OBJECTS':
        return context.selected_objects 

    elif scope == 'ACTIVE_VIEW_LAYER': 
        return context.view_layer.objects

    elif scope == 'ACTIVE_SCENE':
        return context.scene.objects
    
    elif scope == 'ALL_SCENES':    
        return bpy.data.objects

def handle_modifier_visibility(context, copy_settings, scope):
    '''
    Set modifier visibility of all objects within the defined scope
    '''

    for object in get_objects(context, scope):
        for mod in object.modifiers:
            if copy_settings == 'RENDER_TO_VIEWPORT':   
                mod.show_viewport = mod.show_render
            elif copy_settings == 'VIEWPORT_TO_RENDER':
                mod.show_render = mod.show_viewport 

def modifier_has_render_settings(context, modifier):
    '''
    Return True if modifier type has render settings
    '''
    mod_list = ['MULTIRES', 'SUBSURF', 'SCREW', 'OCEAN', 'PARTICLE_SYSTEM']

    return modifier.type in mod_list

def handle_modifier_properties(context, copy_settings, scope):

    '''
    Set modifier properties to copy from view to render or vice versa
    '''
    for object in get_objects(context, scope):
        for mod in object.modifiers:

            if not modifier_has_render_settings(context, mod):
                continue

            if mod.type == 'MULTIRES' or mod.type == 'SUBSURF':
                if copy_settings == 'VIEWPORT_TO_RENDER':
                    mod.render_levels = mod.levels
                else:
                    mod.levels = mod.render_levels 
                
            elif mod.type == 'SCREW':
                if copy_settings == 'VIEWPORT_TO_RENDER':
                    mod.render_steps = mod.steps
                else:
                    mod.steps = mod.render_steps

            elif mod.type == 'OCEAN':
                if copy_settings == 'VIEWPORT_TO_RENDER':
                    mod.resolution = mod.viewport_resolution
                else:
                    mod.viewport_resolution = mod.resolution

        # Particle systems always have render/viewport settings, so loop through all per object
        for particle_system in object.particle_systems:
            if copy_settings == 'VIEWPORT_TO_RENDER':
                particle_system.settings.rendered_child_count = particle_system.settings.child_nbr
            else:
                particle_system.settings.child_nbr = particle_system.settings.rendered_child_count 

def handle_collections_visibility(context, copy_settings, scope):
    '''
    Set collections visibility in one or multiple view layers and scenes based on input from user
    '''
    view_layers = get_view_layers(context, scope)
    
    # Note that hide_viewport is set per view_layer while hide_render is a property that 
    # is set on the collection itself (i.e. not per view layer)
    for view_layer in view_layers:
        for collection in view_layer.layer_collection.children:
            if copy_settings == 'RENDER_TO_VIEWPORT':
                collection.hide_viewport = bpy.data.collections[collection.name].hide_render
            elif copy_settings == 'VIEWPORT_TO_RENDER':
                bpy.data.collections[collection.name].hide_render = collection.hide_viewport



def handle_object_visibility(context, copy_settings, scope):
    
    '''
    Set object visibility in one or multiple view layers and scenes based on input from user
    '''


    if copy_settings == 'RENDER_TO_VIEWPORT':

        view_layers = get_view_layers(context, scope)

        for view_layer in view_layers:
            if scope == 'SELECTED_OBJECTS':
                objects = view_layer.objects.selected
            else:
                objects = view_layer.objects

            # Set visibility setting per object
            for object in objects:
                object = view_layer.objects[object.name]
                object.hide_set(object.hide_render, view_layer = view_layer)


    # Hide render is same for the object in all scenes.
    # Therefore, only loop through active view layer in each scene defined in scope
    elif copy_settings == 'VIEWPORT_TO_RENDER':

        orig_scene = context.scene

        for scene in get_scenes(context, scope):
            
            # Set active scene to scene in order to check which is the active view layer
            context.window.scene = scene
            
            if scope == 'SELECTED_OBJECTS':
                objects = context.view_layer.objects.selected
            else:
                objects = context.view_layer.objects

            for object in objects:
                object = context.view_layer.objects[object.name]
                object.hide_render = object.hide_get()
        
        context.window.scene = orig_scene


def add_menu_button(self, context):
    self.layout.operator(
        OBJECT_OT_visibility_handler.bl_idname,
        text="Visibility handler",
        icon='RESTRICT_VIEW_OFF')

# Registration
def register():
    bpy.utils.register_class(OBJECT_OT_visibility_handler)
    bpy.types.VIEW3D_MT_view.append(add_menu_button)


def unregister():
    bpy.utils.unregister_class(OBJECT_OT_visibility_handler)
    bpy.types.VIEW3D_MT_view.remove(add_menu_button)


if __name__ == "__main__":
    register()
